"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var FiniteAutomaton_1 = __importDefault(require("./FiniteAutomaton"));
var TokenType_1 = __importDefault(require("../Lexer/TokenType"));
var Parser = /** @class */ (function () {
    function Parser(tokens) {
        var _this = this;
        this.tokens = tokens;
        this.parse = function () {
            var syntaxTree = [];
            var finiteAutomaton = new FiniteAutomaton_1.default();
            var queue = [];
            var text = [];
            _this.tokens.forEach(function (token) {
                var state = finiteAutomaton.transit(token);
                switch (state) {
                    case finiteAutomaton.initialState:
                        while (queue.length) {
                            var currToken = queue.shift() || { tokenType: TokenType_1.default.BACKTICK, value: '' };
                            if (currToken.tokenType !== TokenType_1.default.TEXT && currToken.tokenType !== TokenType_1.default.WHITESPACE) {
                                text.push(currToken.tokenType);
                            }
                            else {
                                text.push(currToken.value);
                            }
                        }
                        if (token.tokenType !== TokenType_1.default.TEXT && token.tokenType !== TokenType_1.default.WHITESPACE) {
                            text.push(token.tokenType);
                        }
                        else {
                            text.push(token.value);
                        }
                        break;
                    case finiteAutomaton.leftBraceState:
                        if (token.tokenType === TokenType_1.default.LEFT_BRACE) {
                            while (queue.length) {
                                var currToken = queue.shift() || { tokenType: TokenType_1.default.BACKTICK, value: '' };
                                if (currToken.tokenType !== TokenType_1.default.TEXT && currToken.tokenType !== TokenType_1.default.WHITESPACE) {
                                    text.push(currToken.tokenType);
                                }
                                else {
                                    text.push(currToken.value);
                                }
                            }
                        }
                        queue.push(token);
                        break;
                    case finiteAutomaton.rightBraceState:
                        if (text.length) {
                            syntaxTree.push({
                                content: text.join(''),
                                properties: {}
                            });
                        }
                        text.length = 0;
                        var ruleSetTokens = [];
                        while (queue.length) {
                            var n = queue.shift();
                            if (n) {
                                ruleSetTokens.push(n);
                            }
                        }
                        ruleSetTokens.push(token);
                        syntaxTree.push(_this.getRuleSetNode(ruleSetTokens));
                        break;
                    default:
                        queue.push(token);
                }
            });
            while (queue.length) {
                var currToken = queue.shift() || { tokenType: TokenType_1.default.BACKTICK, value: '' };
                if (currToken.tokenType !== TokenType_1.default.TEXT && currToken.tokenType !== TokenType_1.default.WHITESPACE) {
                    text.push(currToken.tokenType);
                }
                else {
                    text.push(currToken.value);
                }
            }
            if (text.length) {
                syntaxTree.push({
                    content: text.join(''),
                    properties: {}
                });
            }
            return syntaxTree;
        };
        this.getRuleSetNode = function (ruleSetTokens) {
            var finiteAutomaton = new FiniteAutomaton_1.default();
            var ruleSetText = "";
            var ruleSetProperty = "";
            var ruleSetValue = "";
            var ruleSetDeclarations = {};
            ruleSetTokens.forEach(function (ruleSetToken) {
                var ruleSetState = finiteAutomaton.transit(ruleSetToken);
                switch (ruleSetState) {
                    case finiteAutomaton.startTextState:
                        if (ruleSetToken.tokenType !== TokenType_1.default.BACKTICK) {
                            if (ruleSetToken.tokenType !== TokenType_1.default.TEXT && ruleSetToken.tokenType !== TokenType_1.default.WHITESPACE) {
                                ruleSetText += ruleSetToken.tokenType;
                            }
                            else {
                                ruleSetText += ruleSetToken.value;
                            }
                        }
                        break;
                    case finiteAutomaton.propertyState:
                        if (ruleSetToken.tokenType == TokenType_1.default.TEXT) {
                            ruleSetProperty = ruleSetToken.value;
                        }
                        break;
                    case finiteAutomaton.startValueState:
                        if (ruleSetToken.tokenType != TokenType_1.default.BACKTICK) {
                            if (ruleSetToken.tokenType !== TokenType_1.default.TEXT && ruleSetToken.tokenType !== TokenType_1.default.WHITESPACE) {
                                ruleSetValue += ruleSetToken.tokenType;
                            }
                            else {
                                ruleSetValue += ruleSetToken.value;
                            }
                        }
                        break;
                    case finiteAutomaton.endValueState:
                        if (ruleSetToken.tokenType == TokenType_1.default.BACKTICK)
                            ruleSetDeclarations[ruleSetProperty] = ruleSetValue;
                        if (ruleSetToken.tokenType == TokenType_1.default.TEXT)
                            ruleSetDeclarations[ruleSetProperty] = ruleSetToken.value;
                        ruleSetValue = "";
                        break;
                }
            });
            return { content: ruleSetText, properties: ruleSetDeclarations };
        };
    }
    return Parser;
}());
exports.default = Parser;
//# sourceMappingURL=Parser.js.map